# -*- coding: utf-8 -*-
#
# Copyright (C) 2011, 2012, 2013  Cliss XXI
#
# This file is part of Coccinelle.
#
# Coccinelle is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Coccinelle is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Vincent Adolphe
"master django.admin module for this project"

from django import forms
from django.conf import settings
from django.contrib import admin, auth, messages
from django.core import exceptions as dj_execp
from django.urls import reverse

from basket import MixinBasketAdmin, MixinBatchUpdate, MixinBigFilter
from basket import forms as basket_forms
from selectable import forms as select_forms

from coccinelle.contacts import admin_act_csv, admin_act_mail, admin_act_pdf

from ..communes import models as com_models
from ..references import models as ref
from . import lookups as c_lookups
from . import models as c_models


# helper
class PostSaveMsgMixin(object):
    "send message to user after save_related()"

    def save_model(self, request, obj, form, change):
        ret = super(PostSaveMsgMixin, self).save_model(
            request, obj, form, change
        )
        mlist = getattr(form.instance, 'messages', [])
        for msg in mlist:
            messages.add_message(request, *msg)
        return ret


#
# ModelForm
#


class PersonnePhysiqueForm(forms.ModelForm):
    "Hook pour mettre un champs autocomplete"

    class Meta(object):
        model = c_models.PersonnePhysique
        fields = '__all__'


class RelationForm(forms.ModelForm):
    class Meta(object):
        model = c_models.Relation
        fields = '__all__'
        widgets = {
            'structure': select_forms.AutoCompleteSelectWidget(
                lookup_class=c_lookups.StructureLookup
            ),
            'personne': select_forms.AutoCompleteSelectWidget(
                lookup_class=c_lookups.PersonneLookup
            ),
        }

    # Forcer le fait que ce champs soit obligatoire.
    # Cela a ete desactive dans le modele pour permettre
    # au formulaire de modification en masse de ne pas
    # remonter d'erreur si le champs est vide.
    def clean_structure(self):
        if self.cleaned_data["structure"] is None:
            raise dj_execp.ValidationError("Ce champs est obligatoire")
        return self.cleaned_data["structure"]


class StructureForm(forms.ModelForm):
    class Meta(object):
        model = c_models.Structure
        fields = '__all__'
        widgets = {
            'commune': select_forms.AutoCompleteSelectWidget(
                lookup_class=c_lookups.CommuneLookup
            ),
        }


class AbonnementListeForm(forms.ModelForm):
    class Meta(object):
        model = c_models.AbonnementListe
        fields = '__all__'
        widgets = {
            'personne': select_forms.AutoCompleteSelectWidget(
                lookup_class=c_lookups.PersonneLookup
            ),
        }


class PersonnePhysiqueBatch(forms.ModelForm):
    class Meta:
        model = c_models.PersonnePhysique
        fields = ('tag', 'dm_annuaire', 's_perso', 'roleCerdd', 'progCerdd')


class StructureBatch(forms.ModelForm):
    class Meta:
        model = c_models.Structure
        fields = (
            'tag',
            'commune',
            'statut',
        )


class RelationBatch(forms.ModelForm):
    class Meta:
        model = c_models.Relation
        fields = ('fonction', 'titre', 'service', 'structure')

    structure = select_forms.AutoCompleteSelectField(
        lookup_class=c_lookups.StructureLookup,
        required=False,
        help_text="Tapez les premières lettres",
        label="Structure",
    )


#
# Big Filter
#


# Big Filter
class PersonnePhysiqueBFilter(basket_forms.FormBigFilter):
    filtered_qs = dict(
        etiquettes="s_perso__id__in", fonction="relation__fonction__id__in"
    )

    etiquettes = forms.ModelMultipleChoiceField(
        required=False,
        queryset=ref.StatutPerso.objects.all(),
        widget=forms.CheckboxSelectMultiple,
        label="Statut individuel",
    )

    fonction = forms.ModelMultipleChoiceField(
        required=False,
        queryset=ref.Fonction.objects.all(),
        widget=forms.CheckboxSelectMultiple,
        label="Fonction",
    )


# Big Filter
class StructureBFilter(basket_forms.FormBigFilter):
    filtered_qs = dict(
        etiquettes="statut__id__in",
        fonction="relation__fonction__id__in",
        pnr="commune__pnr__id__in",
    )

    etiquettes = forms.ModelMultipleChoiceField(
        required=False,
        queryset=ref.Statut.objects.all(),
        widget=forms.CheckboxSelectMultiple,
        label="Statut",
    )

    fonction = forms.ModelMultipleChoiceField(
        required=False,
        queryset=ref.Fonction.objects.all(),
        widget=forms.CheckboxSelectMultiple,
        label="Fonction",
    )

    pnr = forms.ModelMultipleChoiceField(
        required=False,
        queryset=com_models.PNR.objects.all(),
        widget=forms.CheckboxSelectMultiple,
        label="P.N.R.",
    )


#
# Inlines
#


class AbonnementListeInline(admin.TabularInline):
    model = c_models.AbonnementListe
    exclude = ('demande',)
    readonly_fields = ('status',)
    extra = 0


class RelationInlineS(admin.TabularInline):
    model = c_models.Relation
    extra = 0
    fields = ('fonction', 'personne', 'titre', 'service')
    form = RelationForm


#    readonly_fields = ('service','fonction','titre','personne',)


class RelationInlinePP(admin.TabularInline):
    model = c_models.Relation
    extra = 0
    #    fields = ('structure', 'personne', 'fonction', 'titre', 'service')
    form = RelationForm


#    readonly_fields = ('structure','fonction','titre','personne', 'service')

#
# ModelAdmin
#


class PersonnePhysiqueAdmin(
    PostSaveMsgMixin,
    MixinBatchUpdate,
    MixinBigFilter,
    MixinBasketAdmin,
    admin.ModelAdmin,
):
    # class PersonnePhysiqueAdmin(MixinBatchUpdate, admin.ModelAdmin):
    class Media:
        js = ('basket/js/list_filter_collapsed.js',)

    #    form = PersonnePhysiqueForm
    batch_form = PersonnePhysiqueBatch
    class_form_big_filter = PersonnePhysiqueBFilter
    basket_actions = [admin_act_mail.generate_mailto, 'batchchange_selected']
    actions = basket_actions
    #    list_per_page = 2
    list_display = [
        'id',
        'civilite',
        'nom',
        'prenom',
        'get_list_struct',
        'courriel',
        'tel',
        'mob',
        'get_st_perso',
        'get_tag',
        'comment',
    ]
    # 'dm_annuaire','st_annuaire',
    list_display_link = ['id']
    list_editable = ['civilite', 'nom', 'prenom', 'courriel']
    list_filter = [
        'st_annuaire',
        's_perso',
        'tag',
        'roleCerdd',
        'progCerdd',
        'date_m',
        'date_c',
    ]
    search_fields = [
        'nom',
        'prenom',
        'courriel',
    ]
    #   ce genre de requete fonctionne (mais est certainement plus lourd):
    #    search_fields = ['relation__structure__nom']
    inlines = (AbonnementListeInline, RelationInlinePP)
    readonly_fields = ('st_annuaire', 'date_c', 'date_m')
    #    radio_fields = { 'dm_annuaire' : admin.HORIZONTAL }
    fieldsets = (
        (
            'Dates',
            {
                'classes': ('collapse',),
                'fields': (
                    'date_c',
                    'date_m',
                ),
            },
        ),
        (
            None,
            {
                'fields': (
                    'civilite',
                    'nom',
                    'prenom',
                    'homonymie',
                    'courriel',
                    'tel',
                    'fax',
                    'mob',
                    's_perso',
                )
            },
        ),
        (
            'Cerdd',
            {
                'classes': ('collapse',),
                'fields': (
                    'dm_annuaire',
                    'st_annuaire',
                    'roleRelai',
                    'roleCerdd',
                    'progCerdd',
                ),
            },
        ),
        (
            'Informations complémentaires',
            {
                'classes': ('collapse',),
                'fields': (
                    'comment',
                    'tag',
                ),
            },
        ),
    )  # fin fieldset


class StructureAdmin(
    MixinBigFilter, MixinBatchUpdate, MixinBasketAdmin, admin.ModelAdmin
):
    class Media:
        js = (settings.STATIC_URL + 'basket/js/list_filter_collapsed.js',)

    batch_form = StructureBatch
    class_form_big_filter = StructureBFilter
    basket_actions = [
        admin_act_mail.generate_mailto,
        'batchchange_selected',
        admin_act_pdf.print_label_str,
        admin_act_csv.export_as_csv,
    ]
    actions = basket_actions
    form = StructureForm
    list_display = [
        'id',
        'nom',
        'commune',
        'get_pnr',
        'code_p_eff',
        'courriel',
        'tel',
        'get_tag',
        'comment',
        'adresse',
        'adresse2',
        'code_p',
        'adresse3',
    ]
    list_display_link = [
        'id',
    ]
    list_editable = [
        'nom',
        'commune',
        'courriel',
        'tel',
        'adresse',
        'adresse2',
        'code_p',
        'adresse3',
    ]
    list_filter = [
        'statut',
        'conseil_dev',
        'tag',
        'commune__agglo',
        'commune__pnr',
        'relation__personne__roleCerdd',
        'relation__personne__progCerdd',
    ]
    search_fields = [
        'nom',
        'courriel',
    ]
    readonly_fields = (
        'date_c',
        'date_m',
        'pays',
    )
    inlines = (RelationInlineS,)
    fieldsets = (
        (
            'Dates',
            {'classes': ('collapse',), 'fields': (('date_c', 'date_m'),)},
        ),
        (
            None,
            {
                'fields': (
                    ('nom', 'comment'),
                    ('tel', 'fax'),
                    ('courriel', 'www'),
                    ('statut', 'tag'),
                )
            },
        ),
        (
            'Adresse postale',
            {
                'fields': (
                    'adresse',
                    'adresse2',
                    ('code_p', 'commune', 'adresse3'),
                )
            },
        ),
    )  # fin de fieldset

    def get_changelist_form(self, request, **kwargs):
        kwargs.setdefault('form', StructureForm)
        return super(StructureAdmin, self).get_changelist_form(
            request, **kwargs
        )


class RelationAdmin(MixinBatchUpdate, MixinBasketAdmin, admin.ModelAdmin):
    class Media:
        js = [settings.STATIC_URL + 'basket/js/list_filter_collapsed.js']

    form = RelationForm
    batch_form = RelationBatch
    basket_actions = [
        admin_act_mail.generate_mailto_rel,
        'batchchange_selected',
        admin_act_pdf.print_label_rel,
        admin_act_csv.export_all_relation_as_csv,
    ]
    actions = basket_actions
    list_display = [
        'id',
        'fonction',
        'titre',
        'service',
        'personne_link',
        'get_st_perso',
        'tel',
        'mob',
        'courriel',
        'structure_link',
        'tel_struct',
        'courriel_struct',
    ]
    list_editable = ['fonction', 'titre']
    #    list_editable = [ 'fonction', 'titre' ]
    search_fields = [
        'personne__nom',
        'personne__prenom',
        'personne__courriel',
        'structure__nom',
    ]
    list_filter = [
        'fonction',
        'personne__s_perso',
        'personne__tag',
        'personne__roleCerdd',
        'personne__roleRelai',
        'personne__progCerdd',
        'structure__statut',
        'structure__tag',
        'structure__conseil_dev',
    ]

    def get_changelist_form(self, request, **kwargs):
        kwargs.setdefault('form', RelationForm)
        return super(RelationAdmin, self).get_changelist_form(
            request, **kwargs
        )

    def personne_link(self, request):
        if request.id is None:
            return '-'
        info = (self.admin_site.name, self.opts.app_label, 'personnephysique')
        url = reverse("%s:%s_%s_changelist" % info)
        return "<a href='%s%s/' target='_BLANK'> %s </a>" % (
            url,
            request.personne.id,
            request.personne,
        )

    personne_link.allow_tags = True
    personne_link.short_description = "Personne"

    def structure_link(self, request):
        if request.id is None:
            return '-'
        info = (self.admin_site.name, self.opts.app_label, 'structure')
        url = reverse("%s:%s_%s_changelist" % info)
        return "<a href='%s%s/' target='_BLANK'> %s </a>" % (
            url,
            request.structure.id,
            request.structure,
        )

    structure_link.allow_tags = True
    structure_link.short_description = "Structure"


class AbonnementListeAdmin(admin.ModelAdmin):
    class Media:
        js = [settings.STATIC_URL + 'basket/js/list_filter_collapsed.js']

    form = AbonnementListeForm
    list_display = ['id', 'liste', 'personne', 'status', 'pcourriel']
    list_editable = ['liste', 'personne']
    list_filter = ['liste', 'status']
    fields = ['liste', 'personne', 'status']
    readonly_fields = ['status']

    def get_changelist_form(self, request, **kwargs):
        kwargs.setdefault('form', AbonnementListeForm)
        return super(AbonnementListeAdmin, self).get_changelist_form(
            request, **kwargs
        )


# simplifier l'admin
# admin.site.unregister(auth.models.User)
admin.site.unregister(auth.models.Group)

admin.site.register(c_models.PersonnePhysique, PersonnePhysiqueAdmin)
admin.site.register(c_models.Structure, StructureAdmin)
admin.site.register(c_models.Relation, RelationAdmin)
admin.site.register(c_models.AbonnementListe, AbonnementListeAdmin)
