# -*- coding: utf-8 -*-
#
# Copyright (C) 2011, 2012, 2013  Cliss XXI
#
# This file is part of Coccinelle.
#
# Coccinelle is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Coccinelle is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Vincent Adolphe

# try to keep it generic as possible
"this module manage all csv export"

import csv

from django.core.exceptions import FieldDoesNotExist, PermissionDenied
from django.http import HttpResponse

from . import models as c_models

# TODO: export m2m field
# Model._meta.field[n].name => tout les nom de champs direct du model
# Model._meta.get_all_field_names[n] tout les nom de champs du model
# Model._meta.get_field_by_name(<name>) =>
# (<field_object>, <model_or_None>, <direct-true_or_false>, <m2m-true_or_false>
#
# enumerer les champs direct, et reagir selon m2m ou simple (ordre alpha):
#
# for field, m2m in [ (F2[0], F2[3] ) for F2 in [
#  model._meta.get_field_by_name(F) for F in model._meta.get_all_field_names()
# ] if F2[2] ]:
#     if m2m: pass # champs direct et m2m
#     else: pass # champs direct simple
#
# enumerer selement les m2m (ordre alpha):
# for field in [ F2[0] for F2 in [
#  model._meta.get_field_by_name(F) for F in model._meta.get_all_field_names()
#  ] if (F2[2] and F2[3]) ]:


def get_verbosename(model, fieldname):
    try:
        field = model._meta.get_field(fieldname)
        return field.verbose_name
    except FieldDoesNotExist:
        field = getattr(model, fieldname, None)
        return getattr(field, 'short_description', fieldname)


def getattr_call(obj, field):
    attr = getattr(obj, field)
    return attr() if callable(attr) else attr


def export_as_csv(modeladmin, request, queryset):
    """
    Generic csv export admin action.
    """
    model = queryset.model()
    if hasattr(model, 'nom'):
        queryset = queryset.order_by('nom')
    if not request.user.is_staff:
        raise PermissionDenied
    opts = model._meta
    response = HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename=%s.csv' % str(
        opts
    ).replace('.', '_')
    writer = csv.writer(response)
    field_names = getattr(model, 'csv_fieldnames', None)
    if not field_names:
        field_names = [field.name for field in opts.fields]
    # Write a first row with header information
    writer.writerow(field_names)
    # Write data rows
    for obj in queryset:
        writer.writerow([getattr_call(obj, field) for field in field_names])
    return response


export_as_csv.short_description = "Export CSV limité"


def export_all_relation_as_csv(modeladmin, request, queryset):
    """
    Specific csv export admin action for relation.
    queryset is assumed to be a Relation class.
    """
    queryset = queryset.order_by('personne__nom')
    if not request.user.is_staff:
        raise PermissionDenied
    opts = modeladmin.model._meta
    response = HttpResponse(content_type='text/csv')
    response['Content-Disposition'] = 'attachment; filename=%s.csv' % str(
        opts
    ).replace('.', '_')
    writer = csv.writer(response)

    field_names = [
        (field.model.__name__, field.name, field.verbose_name)
        for field in opts.fields
        if field.name not in ['personne', 'structure']
    ]
    field_names_pp = [
        (field.model.__name__, field.name, field.verbose_name)
        for field in c_models.PersonnePhysique._meta.fields
    ]
    fnames_st = getattr(c_models.Structure, 'csv_fieldnames', None)
    if fnames_st:
        field_names_st = [
            ('structure', fname, get_verbosename(c_models.Structure, fname))
            for fname in fnames_st
        ]
    else:
        field_names_st = [
            (field.model.__name__, field.name, field.verbose_name)
            for field in c_models.Structure._meta.fields
        ]
    writer.writerow(
        ['model:']
        + [n[0] for n in field_names_pp + field_names + field_names_st]
    )
    writer.writerow(
        ['name:']
        + [n[1] for n in field_names_pp + field_names + field_names_st]
    )
    writer.writerow(
        ['verbose_name:']
        + [n[2] for n in field_names_pp + field_names + field_names_st]
    )
    # Write data rows
    for obj in queryset:
        custom_row = [
            '',
        ]
        custom_row.extend(
            [getattr(obj.personne, field[1]) for field in field_names_pp]
        )
        custom_row.extend([getattr(obj, field[1]) for field in field_names])
        custom_row.extend(
            [getattr_call(obj.structure, field[1]) for field in field_names_st]
        )
        writer.writerow(custom_row)
    return response


export_all_relation_as_csv.short_description = "Export CSV Info détaillées"
