# -*- coding: utf-8 -*-
#
# Copyright (C) 2011, 2012, 2013  Cliss XXI
#
# This file is part of Coccinelle.
#
# Coccinelle is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# Coccinelle is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Vincent Adolphe

# try to keep it generic as possible
"this module manage pdf label edition"

import copy
from xml.sax.saxutils import escape

from django.contrib.contenttypes.models import ContentType

# from django.db.models import Count
from django.http import HttpResponse
from django.shortcuts import redirect

from reportlab.lib import units
from reportlab.lib.pagesizes import A4
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.platypus import (
    BaseDocTemplate,
    Frame,
    FrameBreak,
    PageTemplate,
    Paragraph,
)

##
# Impression d'etiquette pdf
##


def print_label_base(modeladmin, request, queryset, get_text, nb_repeat=1):
    "generate a pdf to print labels"
    response = HttpResponse(content_type='application/pdf')
    response['Content-Disposition'] = 'attachment; filename=etiquettes.pdf'

    styleSheet = getSampleStyleSheet()
    style = copy.copy(styleSheet['Normal'])
    style.fontSize = 10

    marge_gauche = 7.2 * units.mm
    marge_haut = 15.2 * units.mm
    marge_interne_gauche = 3 * units.mm
    marge_interne_haut = 1 * units.mm
    marge_inter_hor = 2.5 * units.mm
    marge_inter_ver = 0
    largeur = 63.5 * units.mm
    hauteur = 38.1 * units.mm
    nb_hor = 3
    nb_ver = 7
    pagesize = A4

    frames = []
    for coord_y in range(0, nb_ver):
        for coord_x in range(0, nb_hor):
            frames.append(
                Frame(
                    marge_gauche + coord_x * (largeur + marge_inter_hor),
                    pagesize[1]
                    - marge_haut
                    - coord_y * (hauteur - marge_inter_ver)
                    - hauteur,
                    largeur,
                    hauteur,
                    leftPadding=marge_interne_gauche,
                    rightPadding=0,
                    topPadding=marge_interne_haut,
                    bottomPadding=0,
                    showBoundary=0,
                )
            )
    page_template = PageTemplate(frames=frames)

    doc = BaseDocTemplate(
        response,
        title="Étiquettes",
        pageTemplates=[page_template],
        pagesize=pagesize,
    )

    story = []
    for obj in queryset:
        for i in range(nb_repeat):
            text = get_text(obj)
            text = escape(text)
            text = text.replace("\n", "<br />")
            story.append(Paragraph(text, style))
            story.append(FrameBreak())  # skip to next label
    doc.build(story)

    return response


def get_gen_text(obj):
    "generic base code to be duplicated"
    return str(obj)


def print_label(modeladmin, request, queryset):
    "Imprimer des étiquettes"
    return print_label_base(modeladmin, request, queryset, get_gen_text)


print_label.short_description = "Imprimer des étiquettes"


# code pour les structures
def get_str_text(obj):
    "build text label from a structure object"
    c_post = ""
    if obj.code_p:
        c_post = obj.code_p
    else:
        if obj.commune:
            c_post = obj.commune.postal
    text = "\n".join(
        (
            "",
            obj.nom,
            obj.adresse,
            obj.adresse2,
            " ".join(
                (
                    c_post,
                    obj.commune.nom if obj.commune is not None else '',
                    obj.adresse3,
                )
            ),
        )
    )
    return text


def print_label_str(modeladmin, request, queryset):
    "Imprimer des étiquettes pour les structures"
    return print_label_base(
        modeladmin, request, queryset.order_by('nom'), get_str_text
    )


print_label_str.short_description = (
    "Imprimer des étiquettes pour les structures"
)


# code pour les relation personnes physique / structure
def get_rel_text(obj):
    "build text label from a relation object"
    c_post = ""
    civilite = {'H': 'Monsieur', 'F': 'Madame'}.get(obj.personne.civilite, "")
    commune = ""
    if obj.structure.commune:
        commune = obj.structure.commune.nom
    if obj.structure.code_p:
        c_post = obj.structure.code_p
    else:
        if obj.structure.commune:
            c_post = obj.structure.commune.postal

    text = "\n".join(
        (
            "",
            obj.structure.nom,
            " ".join((civilite, obj.personne.prenom, obj.personne.nom)),
            obj.structure.adresse,
            obj.structure.adresse2,
            " ".join((c_post, commune, obj.structure.adresse3)),
        )
    )
    return text


def print_label_rel(modeladmin, request, queryset):
    """enregistre la qs.query et le contenttype dans la session et
    appelle l'url de gestion d'etiquette"""

    request.session['admin_qs'] = {
        'qsq': 'personne__nom',
        'ctt': ContentType.objects.get_for_model(queryset.model).id,
        'ids': list(queryset.values_list('id', flat=True)),
    }
    return redirect('pdf_check_label')


print_label_rel.short_description = (
    "Imprimer des étiquettes pour les personnes"
)
