# -*- coding: utf-8 -*-
# contacts
# C-x RET C-\ set-input-method latin-1-prefix
from urllib.error import URLError

from django.contrib import messages
from django.db import models
from django.urls import reverse

from suds import WebFault

from ..communes import models as com
from ..references import models as ref
from .cocci_sympa import CocciSympa

# Create your models here.

CHOIX_CIVILITE = (
    ('F', 'Madame'),
    #                   (u'',        'Non renseigné'),
    ('H', 'Monsieur'),
)

CHOIX_ANNUAIRE = (
    (0, 'Ne rien faire'),
    (1, 'Ajouter'),
    (2, 'Enlever'),
)

STATUT_ANNUAIRE = (
    (0, 'Inconnu'),
    (1, 'inscrit'),
    (2, 'non inscrit'),
)

CHOIX_ABO = (
    (0, 'Ne rien faire'),
    (1, 'Désabonner'),
    (2, 'Abonner'),
)

STATUT_ABO = (
    (0, 'Inconnu'),
    (1, 'Abonné'),
    (2, 'Non Abonné'),
    (3, 'Désabonné par l\'utilisateur ou par sympa'),
    (4, 'Liste inconnue sur sympa, abonnement demandé'),
    (5, 'Liste inconnue sur sympa, désabonnement demandé'),
)


ST_SYM = 0b1000
ST_USR = 0b0100
ST_SUB = 0b0010
ST_ERR = 0b0001
SUB_STATUS = (
    #  +--- ST_SYM status from sympa (0: no, 1: yes)
    #  |+-- ST_USR status from user (0: cocci operator, 1: user)
    #  ||+- ST_SUB subscribe status (0: unsubscribe, 1: subscribe)
    #  |||+ ST_ERR error status (0: no error, 1: error on sympa side)
    (0b0000, "inconnu"),  # desabonne par admin = sup. abo
    (0b0010, "abonné via coccinelle"),  # abonné par l'admin
    (0b0011, "abonné via coccinelle - err dans sympa"),  # abonné par l'admin
    (0b0100, "l'utilisateur s'est désabonné"),
    (0b0101, "l'utilisateur s'est désabonné - err dans sympa"),
    (0b0110, "l'utilisateur s'est abonné"),
    (0b0111, "l'utilisateur s'est abonné - err dans sympa"),
    (0b1000, "désabonné sur sympa (bounce?)"),
)
DSUB_STATUS = dict(SUB_STATUS)

ERR_BAD_CNX = "Connexion au serveur sympa impossible"


def init_sympa_cnx():
    error = None
    try:
        sympa = CocciSympa()
        sympa.login()
    except URLError:
        error = ERR_BAD_CNX
    except WebFault as err:
        error = err.fault['detail']
    return sympa, error


# pour les champs en commun entre PersonnePhysique et Structure
# hors des Foreign et des ManyToMany
class ChampsCommun(models.Model):
    class Meta:
        abstract = True

    tel = models.CharField("Téléphone", max_length=20, blank=True)
    courriel = models.EmailField("Courriel", max_length=100, blank=True)
    fax = models.CharField(max_length=20, blank=True)
    comment = models.CharField("Commentaire", max_length=400, blank=True)
    date_c = models.DateField("Date création", blank=True, auto_now_add=True)
    date_m = models.DateField("Date modification", blank=True, auto_now=True)


class PersonnePhysique(ChampsCommun):
    class Meta:
        verbose_name = "Personne physique"
        verbose_name_plural = "   Personnes physiques"
        unique_together = ('nom', 'prenom', 'homonymie')
        ordering = ('nom',)

    messages = []
    civilite = models.CharField(
        "Civilité", max_length=5, choices=CHOIX_CIVILITE, blank=True
    )
    nom = models.CharField(max_length=100)
    prenom = models.CharField("Prénom", max_length=100, blank=True)
    homonymie = models.CharField(
        max_length=100,
        blank=True,
        help_text="Permet de différencier une personne en cas d'homonymie",
    )
    mob = models.CharField("Mobile", max_length=20, blank=True)
    tag = models.ManyToManyField(
        ref.TagPersonnePhysique, verbose_name="Étiquette(s)", blank=True
    )
    dm_annuaire = models.IntegerField(
        "Inscription annuaire", choices=CHOIX_ANNUAIRE, default=0
    )
    st_annuaire = models.IntegerField(
        "Statut annuaire", editable=False, choices=STATUT_ABO, default=0
    )
    s_perso = models.ManyToManyField(
        ref.StatutPerso, verbose_name="Statut Personnel", blank=True
    )
    roleCerdd = models.ManyToManyField(
        ref.CerddCentral, verbose_name="Rôle au sein du Cerdd", blank=True
    )
    roleRelai = models.BooleanField(verbose_name="Rôle relai", default=False)
    progCerdd = models.ManyToManyField(
        ref.CerddProgramme, verbose_name="Programme Cerdd", blank=True
    )
    # AbonnementListe pointe sur PersonnePhysique

    def __str__(self):
        if self.homonymie:
            return "%s %s (%s)" % (self.prenom, self.nom, self.homonymie)
        return "%s %s" % (self.prenom, self.nom)

    def get_st_perso(self):
        return ", ".join([i_st.nom for i_st in self.s_perso.all()])

    get_st_perso.short_description = "Statut perso"

    def get_tag(self):
        return ", ".join([i_st.nom for i_st in self.tag.all()])

    get_tag.short_description = "Étiquette(s)"

    def get_list_struct(self):
        return " / ".join(
            [rel.structure.nom for rel in self.relation_set.all()]
        )

    get_list_struct.short_description = "Structure(s)"

    def get_absolute_url(self):
        info = self._meta.app_label, self._meta.model_name
        return reverse("admin:%s_%s_change" % info, args=(self.id,))

    def clean(self):
        self.nom = self.nom.upper()

    def save(self, upd_sympa=True, force_upd=False, **kwargs):
        if not upd_sympa:
            return super(PersonnePhysique, self).save(**kwargs)
        if self.pk is not None:  # maj
            old_pp = PersonnePhysique.objects.get(pk=self.pk)
            force_upd = force_upd or (old_pp.courriel != self.courriel)
        super(PersonnePhysique, self).save(**kwargs)
        if force_upd:
            sympa, error = init_sympa_cnx()
            if error:
                self.messages.append([messages.WARNING, error])
            else:
                for abo in self.abonnementliste_set.all():
                    if (abo.status & ST_SUB) or not abo.status:
                        sympa.replace(
                            abo.liste.adresse,
                            old_pp.courriel,
                            self.courriel,
                            force=True,
                        )
                        if abo.status & ST_USR:
                            abo.status &= ~ST_USR
                            abo.save(upd_sympa=False)


class Structure(ChampsCommun):
    class Meta:
        verbose_name_plural = "  Structures"
        unique_together = ('nom', 'commune')
        ordering = ('nom',)

    csv_fieldnames = [
        'nom',
        'adresse',
        'adresse2',
        'adresse3',
        'commune',
        'code_p_eff',
        'statut',
        'conseil_dev',
        'www',
    ]
    nom = models.CharField(max_length=100, verbose_name="Raison sociale")
    adresse = models.CharField(
        max_length=100, verbose_name="Adresse", blank=True
    )
    adresse2 = models.CharField(
        max_length=100, verbose_name="Adresse (suite)", blank=True
    )
    adresse3 = models.CharField(
        max_length=100,
        verbose_name="Adresse (cedex)",
        blank=True,
        help_text="cette partie de l'adresse apparaitra apres "
        "le code postal et la ville",
    )
    commune = models.ForeignKey(
        com.Commune,
        verbose_name="Commune",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
    )
    code_p = models.CharField(
        max_length=15,
        verbose_name="Code Postal",
        blank=True,
        help_text="préciser le code postal si différent de celui de la commune",
    )
    statut = models.ForeignKey(
        ref.Statut,
        verbose_name="Statut",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
    )
    conseil_dev = models.BooleanField(
        verbose_name="Conseil de développement", default=False
    )
    tag = models.ManyToManyField(
        ref.TagStructure, verbose_name="Étiquette(s)", blank=True
    )
    www = models.CharField(
        max_length=200, verbose_name="Site internet", blank=True
    )

    def __str__(self):
        if self.commune:
            return "%s (%s)" % (self.nom, self.commune.nom)
        return "%s" % (self.nom)

    def pays(self):
        if self.commune:
            return self.commune.pays

    def code_p_eff(self):
        if self.code_p:
            return self.code_p
        elif self.commune and self.commune.postal:
            return self.commune.postal
        return ""

    code_p_eff.short_description = "Code postal"

    def get_pnr(self):
        if self.commune and self.commune.pnr:
            return self.commune.pnr
        return ""

    get_pnr.short_description = "P.N.R."

    def get_tag(self):
        return ", ".join([i_st.nom for i_st in self.tag.all()])

    get_tag.short_description = "Étiquette(s)"

    def get_absolute_url(self):
        info = self._meta.app_label, self._meta.model_name
        return reverse("admin:%s_%s_change" % info, args=(self.id,))


class Relation(models.Model):
    class Meta:
        verbose_name = " Info détaillée (personne)"
        verbose_name_plural = " Infos détaillées (personne)"
        unique_together = ('personne', 'structure')
        ordering = ('personne',)

    personne = models.ForeignKey(PersonnePhysique, on_delete=models.CASCADE)
    fonction = models.ForeignKey(
        ref.Fonction,
        verbose_name="Fonction abrégée",
        blank=True,
        null=True,
        on_delete=models.PROTECT,
    )
    titre = models.CharField(
        max_length=100,
        verbose_name="Fonction",
        help_text="utilisé pour le publipostage",
        blank=True,
    )
    structure = models.ForeignKey(
        Structure, blank=True, null=True, on_delete=models.CASCADE
    )
    service = models.CharField(
        max_length=100, verbose_name="Service", blank=True
    )

    def __str__(self):
        return "%s, %s, %s" % (self.personne, self.fonction, self.structure)

    def tel(self):
        return self.personne.tel

    tel.short_description = "Tél (pers.)"

    def mob(self):
        return self.personne.mob

    mob.short_description = "Mobile (pers.)"

    def courriel(self):
        return self.personne.courriel

    courriel.short_description = "Courriel (pers.)"

    def tel_struct(self):
        return self.structure.tel

    tel_struct.short_description = "Tél. (struct.)"

    def courriel_struct(self):
        return self.structure.courriel

    courriel_struct.short_description = "Courriel (struct.)"

    def get_st_perso(self):
        return self.personne.get_st_perso()

    get_st_perso.short_description = "Statut perso"

    def get_absolute_url(self):
        info = self._meta.app_label, self._meta.model_name
        return reverse("admin:%s_%s_change" % info, args=(self.id,))


class AbonnementListe(models.Model):
    class Meta:
        verbose_name = "Abonnement liste de diffusion"
        verbose_name_plural = "Abonnements listes de diffusion"

    messages = []
    personne = models.ForeignKey(PersonnePhysique, on_delete=models.CASCADE)
    liste = models.ForeignKey(ref.Liste, on_delete=models.PROTECT)
    demande = models.IntegerField(
        "Demande d'inscription", choices=CHOIX_ABO, default=0
    )
    status = models.IntegerField(
        "Statut inscription", choices=SUB_STATUS, default=0, editable=False
    )

    def __str__(self):
        return "[%s] %s" % (self.liste, self.personne)

    def pcourriel(self):
        return self.personne.courriel

    pcourriel.short_description = "Courriel"

    def save(self, upd_sympa=True, from_user=False):
        if not upd_sympa:
            return super(AbonnementListe, self).save()

        sympa, error = init_sympa_cnx()
        if error:
            self.messages.append([messages.WARNING, error])
            return super(AbonnementListe, self).save()

        if self.pk is not None:  # maj: sup. ancien abo
            old_abo = AbonnementListe.objects.get(pk=self.pk)
            if (
                old_abo.liste.adresse != self.liste.adresse
                or old_abo.personne.courriel != self.personne.courriel
            ):
                sympa.rem(
                    old_abo.liste.adresse,
                    old_abo.personne.courriel,
                    force=True,
                )
            else:  # maj mais pas de changement apparent
                pass
        else:  # creation
            if self.status == 0:  # creation n'impliquant pas un utilisateur
                self.status = ST_SUB
        # creation ou mise a jour, creer le nouvel abonnement si besoin
        try:
            if self.status & ST_SUB:
                sympa.add(self.liste.adresse, self.personne.courriel)
            else:
                sympa.rem(self.liste.adresse, self.personne.courriel)
        except WebFault as err:
            if err.fault.faultstring != "Unable to add user":  # deja inscrit
                self.status |= ST_ERR
        super(AbonnementListe, self).save()

    def delete(self):
        sympa, error = init_sympa_cnx()
        if error:
            self.messages.append([messages.WARNING, error])
        else:
            try:
                sympa.rem(self.liste.adresse, self.personne.courriel)
            except WebFault as err:
                self.messages.append([messages.WARNING, err.fault['detail']])
        super(AbonnementListe, self).delete()


# EOF
