from unittest.mock import MagicMock, patch
from urllib.error import URLError

import pytest
from suds import WebFault

from coccinelle.contacts.models import (
    ERR_BAD_CNX,
    PersonnePhysique,
    Relation,
    Structure,
    init_sympa_cnx,
)
from coccinelle.references.models import (
    StatutPerso,
    TagPersonnePhysique,
    TagStructure,
)


@pytest.mark.django_db
def test_personnephysique_rel_obj():
    st_perso1 = StatutPerso.objects.create(nom="statut1")
    tag1 = TagPersonnePhysique.objects.create(nom="tag1")
    struct1 = Structure.objects.create(nom="struct1")
    pers1 = PersonnePhysique.objects.create(nom="pers1")
    Relation.objects.create(personne=pers1, structure=struct1)
    pers1.s_perso.set([st_perso1])
    assert pers1.get_st_perso() == "statut1"
    pers1.tag.set([tag1])
    assert pers1.get_tag() == "tag1"
    assert pers1.get_list_struct() == "struct1"


@pytest.mark.django_db
def test_structure_rel_obj():
    tag1 = TagStructure.objects.create(nom="tag1")
    struct1 = Structure.objects.create(nom="struct1")
    struct1.tag.set([tag1])
    assert struct1.get_tag() == "tag1"


@pytest.mark.django_db
def test_relation_rel_obj():
    pers1 = PersonnePhysique.objects.create(nom="pers1")
    st_perso1 = StatutPerso.objects.create(nom="statut1")
    pers1.s_perso.set([st_perso1])
    rel1 = Relation(personne=pers1)
    assert rel1.get_st_perso() == "statut1"


@pytest.mark.parametrize(
    'exception,exp_msg',
    [
        (None, None),
        (URLError('yolo'), ERR_BAD_CNX),
        (WebFault({'detail': 'plouf'}, 'doc'), 'plouf'),
    ],
)
@patch('coccinelle.contacts.cocci_sympa.CocciSympa.__new__')
def test_init_sympa_cnx(cocci_class, exception, exp_msg):
    mock_inst = MagicMock()
    if exception:
        mock_inst.login.side_effect = exception
    cocci_class.return_value = mock_inst
    sympa, error = init_sympa_cnx()
    assert error == exp_msg


@patch('coccinelle.contacts.cocci_sympa.CocciSympa.__new__')
class TestPersonnePhysiqueSave:
    @patch('django.db.models.Model.save')
    def test_personnephysique_save_no_upd_sympa(self, dj_save, cocci_class):
        pers1 = PersonnePhysique(nom="pers1")
        pers1.save(upd_sympa=False)
        dj_save.assert_called_with()

    @pytest.mark.django_db
    @pytest.mark.parametrize('email_change', [False, True])
    def test_personnephysique_save(self, cocci_class, email_change):
        mock_inst = MagicMock()
        cocci_class.return_value = mock_inst
        mock_inst.login.side_effect = URLError('yolo')
        pers1 = PersonnePhysique(nom="pers1", courriel="pers1@no.lan")
        pers1.save(upd_sympa=False)
        if email_change:
            pers1.courriel = "pers1@next.lan"
        pers1.save(upd_sympa=True, force_upd=False)
